'use strict';
//templater for sink commands
var path = require("path")
var fs = require("fs");
'use strict';
var nmea = require('nmea');
var EventEmitter2 = require("eventemitter2").EventEmitter2;

const merger = (prefix, thing) => (gather, key) => {
  gather[prefix+key] = Array.isArray(thing) ? thing[idx] : thing[key];
  return gather;
}
const expandCDC = module.exports = (obj) => {
  try {
    if (obj.usb_cdc.indexOf("GPGGA") >= 0){
      var gps = nmea.parse(obj.usb_cdc);
      console.log("got gps", gps)
      return Object.keys(gps).reduce(merger("gps_", gps), obj)
    }
    switch(obj.usb_cdc.split(",")[0]){
      case "getCurrent":
        (["time","bus_voltage","shunt_voltage","load_voltage","current","amp_hours","watt_hours"]).reduce(merger("current_",obj.usb_cdc.split(',').slice(1)) ,obj)
      case "peopleCounter":
        (["human_count","distance"]).reduce(merger("people_", obj.usb_cdc.split(',').slice(1)), obj)
      default :
        return obj
    }
  } catch (e) {
    console.log(e);
    return obj;
  }
}
//seperate

// turn "-i,--inline <stuff>" => "inline"
const optkey = (opt) => opt[0].split(",--")[1].split(" ")[0];

const wildcard = (events, label) => (events.indexOf("*") > -1 && events.indexOf("-" + label) < 0)

const explicit = (events, label) => (events.indexOf(label) > -1)

const eventEnabled = (events, label) => (wildcard(events, label) || explicit(events,label))

const flattenReport = (obj) => Object.keys(obj).reduce((nobj, key) => {
  if (key == "usb"){
    Object.keys(obj[key]).forEach((kk) => {
      nobj["usb_" + kk] = obj[key][kk];
    })
  } else {
    nobj[key] = obj[key]
    if (typeof nobj[key] == 'string'){
       nobj[key] = nobj[key]
    }
  }
  return nobj
}, {})

const unpackReport = (array) => {
  try {
    let keys = ["hn","up","temp","humidity","lux","movex","movey","movez","dB","rssi","usb"];
    return keys.reduce((rep, key, index) => {
      if (key == "movex")
        rep["move"] = array.slice(index, index + 3).join(',');
      else if (key == "usb")
        rep["usb"] = {cdc : array[index] || undefined}
      else if (key == "movey" || key == "movez")
        return rep;
      else
        rep[key] = array[index];
      return rep;
    }, {});
  } catch (E) {
    return array;
  }
}



const normalize = (evt) => {
  try {
    let data = JSON.parse(evt.data);
    if (Array.isArray(data))
      data = unpackReport(data);
    if (typeof data == "object")
      data = flattenReport(data);
    if (data.usb_cdc)
      data = expandCDC(data);

    evt.data = data;
    evt.data.via = evt.via;
  } catch (e) {}

  return evt;
}

const invalid = (args, options) =>
  options.reduce((res, opt) =>
    typeof res == "string"       ? res
    : !args.options[optkey(opt)] && !opt[2] ? "missing option " + optkey(opt)
    : false
  , false);



const commandTemplate = module.exports = (p) => (args) => (Mesh, th) => {
  try {

  require(path.join(__dirname, p))((name, options, setup) => {
    args = args || {};
    args.options = args.options || {};
    let error = invalid(args, options);
    if (error) throw new Error(error);
    if (!Mesh._sink_events) {
      Mesh._sink_events = new EventEmitter2();
      Mesh.on('open' , (link, open) => {
        if (open.json && open.json.type == "event"){
          Mesh._sink_events.emit(open.json.label, open.json)
        }
      })
    }

    options.forEach((opt) => {
      if (opt[2] && !args.options[optkey(opt)]) args.options[optkey(opt)] = opt[2];
    })

    var events = (args.events || "*").split(",")
    setup(args.options, (runner) => {
      Mesh._sink_events.onAny((label, evt) => {
        try{
          if (eventEnabled(events, label))
            runner(label, normalize(evt));
        } catch (e){
          console.log(`error in ${name} sink`,e);
        }
      })
    })
  })
} catch (e){
  console.error(e);
} finally {
  return {};
}

} ;
